/* Program memory
 * Michael Robinson 
 */
`timescale 1ns/10ps

/* Program memory, instruction decode, branch control
 * Input:
 *        sre_ = When halted, switches refer to state register
 *        mee_ = When halted, switches refer to memory/devices
 *               Used to tell when this device should not be placing
 *               an address on ablo
 *        i1e_ = When halted, switches refer to first instruction word
 *        i2e_ = When halted, switches refer to second instruction word
 *        i3e_ = When halted, switches refer to third instruction word
 *        run, run_ = run switch state (run=high -> machine running)
 *        write_ = active low write to whichever memory is selected by
 *                 (sre_, mee_, i1e_, i2e_, i3e_)
 *        ensr_ = allow state register on aiprg bus
 *        aim1 = branch selection bit from address switches
 *        t1 = timestate 1: store condition bit
 *        t4 = timestate 4: branch to next location
 *        f  = condition bit
 *        reset_ = system-wide reset
 * Output:
 *        [1:0] dest = destination of alu operation
 *        [3:0] op = operation code for alu
 *        i = indirection bit
 * Input/Output:
 *        [7:0] aiprg = program address bus
 *        [7:0] ablo = low byte of data address bus and second 
 *                     instruction word
 *        [7:0] dii1 = connection to first instruction word
 *        [7:0] dii3 = connection to third instruction word
 */
module progmem ( sre_, i1e_, i2e_, i3e_, mee_, run, run_, write_,
                 ensr_, aim1, t1, t4, f, aiprg, ablo, dest, op, i,
                 dii1, dii3, reset_ );
input sre_, i1e_, i2e_, i3e_, mee_;
input run, run_, write_;
input ensr_, aim1, reset_;

input t1, t4, f;

inout [7:0] aiprg;
inout [7:0] ablo;
inout [7:0] dii1, dii3;

output i;
output [1:0] dest;
output [3:0] op;

wire i;
wire [1:0] dest;
wire [3:0] op;

wire brbit;  /* Branch select bit */
wire f_stored;
wire srclk;  /* Clock line for state register */
wire i1oe_;
wire i1we_;
wire i2oe_;
wire i2we_;
wire i3oe_;
wire i3we_;

assign brbit = ( f_stored & run ) | ( aim1 & run_ );

/* SR is written on rising edge of write_, reverse of most other
 * things.  Further, machine must be reset before SR will take a
 * manual write. */
assign srclk = t4 & ( sre_ | write_ );
assign i1oe_ = ~ i1e_;
assign i1we_ = write_ | run | i1e_;
assign i2oe_ = ~ ( mee_ & i2e_ & ~( i & run ) );
assign i2we_ = write_ | run | i2e_;
assign i3oe_ = ~( i3e_ & sre_ );
assign i3we_ = write_ | run | i3e_;
assign i = dii1[0];
assign dest = dii1[2:1];
assign op = dii1[6:3];

u74ls74 f_bit( .d(f), .clk(t1), .q(f_stored), .s_(1), .r_(reset_) );

u74ls374 sr( .d(dii3), .clk(srclk), .q(aiprg), .oe_(ensr_) );

mem13 i1( .d(dii1), .a({5'b00000,aiprg}), .ce_(0), .oe_(i1oe_),
          .we_( i1we_ ) ); 

mem13 i2( .d(ablo), .a({5'b00000,aiprg}), .ce_(0), .oe_(i2oe_),
          .we_( i2we_ ) );

mem13 i3( .d(dii3), .a({4'b0000,brbit,aiprg}), .ce_(0), .oe_(i3oe_),
          .we_( i3we_ ) );
endmodule